package server.command.commands;

import java.security.MessageDigest;

import common.color.TextColor;
import common.network.IPlayer;
import common.util.EncryptUtil;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.command.UserPolicy;
import server.network.Player;
import server.network.User;
import server.util.Form;
import server.vars.SVars;

public class CommandPasswd extends Command {
	public CommandPasswd() {
		super("passwd");

		this.addUser("user", true, UserPolicy.NON_ADMINS_OR_SELF);
		this.setParamsOptional();
		this.addString("password", false);
	}
	
	public void exec(CommandEnvironment env, Executor exec, User user, String password) {
		if(exec.isPlayer()) {
			if(password != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Passwort angegeben werden");
			if(user.getPubkey() != null && user == exec)
				throw new RunException("Es darf kein Pubkey vorhanden sein, um diesen Befehl an sich selbst anzuwenden");
			if(user.getPasswordHash() == null && user == exec)
				throw new RunException("Es muss ein Passwort vorhanden sein, um diesen Befehl an sich selbst anzuwenden");
			((Player)exec).displayForm(new Form() {
				private Field checkField;
				private Field passwordField;
				private Field confirmField;
				
				protected void init() {
					this.checkField = user != exec ? null : this.addPassword("Aktuelles Passwort", 0, IPlayer.MAX_PASS_LENGTH, "");
					this.passwordField = this.addPassword("Neues Passwort", SVars.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
					this.confirmField = this.addPassword("Passwort bestätigen", SVars.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
				}
				
				public String getTitle() {
					return "Passwort für " + user.getUser() + " ändern";
				}
				
				protected void accept() {
					User plr = env.getServer().getUser(user.getUser());
					if(!((Player)exec).isAdmin() || plr == null || (plr.isAdmin() && plr != exec) || (plr.getPasswordHash() == null && plr == exec) || (plr.getPubkey() != null && plr == exec)) {
						exec.logConsole(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					if(this.checkField != null && !MessageDigest.isEqual(EncryptUtil.hashPassword(this.checkField.get(), plr.getPasswordHash().second()), plr.getPasswordHash().first())) {
						exec.logConsole(TextColor.RED + "Falsches Passwort eingegeben");
						return;
					}
					if(!this.passwordField.get().equals(this.confirmField.get())) {
						exec.logConsole(TextColor.RED + "Passwörter stimmen nicht überein");
						return;
					}
					plr.setPasswordHash(EncryptUtil.hashPassword(this.passwordField.get()));
					plr.setPubkey(null);
					exec.logConsole(TextColor.GREEN + "Passwort" + (plr != exec ? " für %s" : "") + " gesetzt", plr.getUser());
				}
			});
		}
		else if(exec.isConsole()) {
			if(password == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Passwort angegeben werden");
			if(password.length() < 8)
				throw new RunException("Das Passwort ist zu kurz, mindestens 8 Zeichen sind erforderlich");
			user.setPasswordHash(EncryptUtil.hashPassword(password));
			user.setPubkey(null);
			exec.logConsole(TextColor.GREEN + "Passwort für %s gesetzt", user.getUser());
		}
	}
}
