package server.command.commands;

import common.color.TextColor;
import common.network.IPlayer;
import common.util.EncryptUtil;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.network.User;
import server.util.Form;
import server.vars.SVars;

public class CommandRegister extends Command {
	public CommandRegister() {
		super("register");

		this.addString("username", false, null, IPlayer.MAX_USER_LENGTH, IPlayer.VALID_USER);
		this.setParamsOptional();
		this.addString("password", false);
		this.addFlag("admin", 'a');
	}
	
	public void exec(CommandEnvironment env, Executor exec, String username, String password, boolean admin) {
		if(env.getServer().getUser(username) != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen ist bereits registriert");
		if(env.getServer().loadPlayerData(username) != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen hat bereits Spielerdaten, ist aber nicht registriert");
		if(exec.isPlayer()) {
			if(password != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Passwort angegeben werden");
			((Player)exec).displayForm(new Form() {
				private Field passwordField;
				private Field confirmField;
				
				protected void init() {
					this.passwordField = this.addPassword("Passwort", SVars.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
					this.confirmField = this.addPassword("Passwort bestätigen", SVars.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
				}
				
				public String getTitle() {
					return "Spieler " + username + " registrieren";
				}
				
				protected void accept() {
					if(!((Player)exec).isAdmin() || env.getServer().getUser(username) != null || env.getServer().loadPlayerData(username) != null) {
						exec.logConsole(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					if(!this.passwordField.get().equals(this.confirmField.get())) {
						exec.logConsole(TextColor.RED + "Passwörter stimmen nicht überein");
						return;
					}
					User user = new User(username);
					user.setPasswordHash(EncryptUtil.hashPassword(this.passwordField.get()));
					user.setAdmin(admin);
					env.getServer().addUser(user);
					exec.logConsole(TextColor.GREEN + "Spieler %s registriert", username);
				}
			});
		}
		else if(exec.isConsole()) {
			if(password == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Passwort angegeben werden");
			if(password.length() < 8)
				throw new RunException("Das Passwort ist zu kurz, mindestens 8 Zeichen sind erforderlich");
			User user = new User(username);
			user.setPasswordHash(EncryptUtil.hashPassword(password));
			user.setAdmin(admin);
			env.getServer().addUser(user);
			exec.logConsole(TextColor.GREEN + "Spieler %s registriert", username);
		}
	}
}
